header
{
/**
 * Semantic analysis for the Mile Language.
 * 
 * @author Ricardo Soto
 * @since 1.5
 */

package cl.ucv.inf.mileppcompiler.compilers;

}

  
class MilePPCodeGen extends TreeParser;
options
{
    importVocab=MilePPParserVocab;
    buildAST = false;
    defaultErrorHandler=false;
}

{
private CodeGenerator   cG = new CodeGenerator();
private String classId = "";

	public void setId(AST id) {
		this.classId = id.toString();
	}
	
	public String getId() {
		return classId;
	}
}


//==================================================
//Program

program  : #(PROGRAM main_class_dec (class_dec)* {cG.end();});


//======================================================= 
//Class declarations
        
main_class_dec : #(MAIN_RW class_dec);


class_dec : #(CLASS_RW id1:IDENT {cG.addClass(id1);this.setId(id1);}
             (EXTENDS_RW IDENT)? 
             (param)? class_body {cG.endClass();});

param :  #(PARAM type IDENT (COMMA type IDENT)*);
         
class_body : #(BODY var_dec method_dec);


//=======================================================
// Var dec

var_dec : #(VAR_DEC (basic_var_dec| object_dec)*);

basic_var_dec : #(BASIC_VAR_DEC basic_type IDENT);

object_dec : #(OBJECT_RW IDENT IDENT);
          
type:     basic_type|IDENT;

basic_type:   (NUMERIC_TYPE|STRING_TYPE) ;

//=======================================================
// Method dec

method_dec : #(METHOD_RW (t:type id2:IDENT {cG.addMethod(t,id2);}  (param)? var_dec body return_st {cG.endMethod();})*);
              
return_st : #(RETURN_RW {cG.addReturn();} expr {cG.endReturn();} );

//=======================================================
// Body

body : #(BODY (assign)*);
        
         
//=======================================================
// Assignment

assign : #(ASSIGN id1:IDENT ASSIG {cG.addAssign(id1);} expr 
           {cG.endAssign();});
        
                 
//=================================================
//Expressions
expr
  : #(OR_RW           expr expr) 
  | #(AND_RW          expr expr) 
  | #(EQUAL           expr expr) 
  | #(GR_EQUAL        expr expr)  
  | #(NOT_EQUAL       expr expr) 
  | #(GR              expr expr) 
  | #(LE              expr expr) 
  | #(PLUS            expr {cG.addPlus();} expr) 
  | #(SUB             expr expr) 
  | #(STAR            expr expr) 
  | #(SLASH           expr expr) 

  | #(UN_MINUS                   expr)
  | #(RES_NOT                    expr) 
  | IDENT 
  | lit
  | access
  ;

  lit: r:REAL_LITERAL {cG.addNumber(r);}|i:INT_LITERAL {cG.addNumber(i);};
  
  access : #(ACCESS (id:IDENT {cG.addIdent(this.getId(),id);}(DOT IDENT | access_param)*));

  access_param: #(ACCESS_PARAM (IDENT (COMMA IDENT)*)? );
  
  