header
{
/**
 * Parser for the Mile language.
 * 
 * @author Ricardo Soto
 * @since 1.5
 */

package cl.ucv.inf.mileppcompiler.compilers;

}
//======================================================
// Object allowing syntactic analysis

class MilePPParser extends Parser;
options
{
    k = 2;
    importVocab = MilePPLexerVocab;
    exportVocab = MilePPParserVocab;
    buildAST = true;
    defaultErrorHandler=false;
}


tokens
{
//=======================================================
// Imaginary Tokens 


    PROGRAM;
    VAR_DEC;
    UN_MINUS;
    BODY;
    ASSIGN;
    FOR_ST;
    IF_ST;
    PARAM;
    ELSE_ST;
    FOR_HEADER;
    BASIC_VAR_DEC;
    ACCESS;
    ACCESS_PARAM;


}

//======================================================= 
//Main rule

program : main_class_dec (class_dec)*
          {## = #( #[PROGRAM, "PROGRAM"] ,##);}
        ;
        

//======================================================= 
//Class declarations
        
main_class_dec : MAIN_RW^ class_dec;

class_dec : CLASS_RW^ IDENT (EXTENDS_RW IDENT)? 
            LEFT_PAREN! (param)? RIGHT_PAREN!
            LEFT_BRACE! class_body RIGHT_BRACE!;

param :  type IDENT (COMMA type IDENT)* 
         {## = #( #[PARAM, "PARAM"] ,##);};
                 
basic_type:   (NUMERIC_TYPE|STRING_TYPE) ;



//=======================================================
// Var dec

var_dec : (basic_var_dec| object_dec)* 
          {## = #( #[VAR_DEC, "VAR_DEC"] ,##);};

basic_var_dec : basic_type IDENT SEMICOLON! 
          {## = #( #[BASIC_VAR_DEC, "BASIC_VAR_DEC"] ,##);};

object_dec : OBJECT_RW^ IDENT IDENT SEMICOLON!;
          
type:     basic_type|IDENT;


//=======================================================
// Method dec

method_dec : (METHOD_RW^ type IDENT LEFT_PAREN! 
             (param)? RIGHT_PAREN! LEFT_BRACE! var_dec body return_st 
              RIGHT_BRACE!)*;
              
return_st : RETURN_RW^ expr SEMICOLON!;



//=======================================================
// Body

class_body : var_dec method_dec
       {## = #( #[BODY, "BODY"] ,##);};

body : (assign |st | print | read)*
       {## = #( #[BODY, "BODY"] ,##);};
       
st : (for_st|if_st);

//=======================================================
// Assignment

assign : IDENT ASSIG expr SEMICOLON!
        {## = #( #[ASSIGN, "ASSIGN"] ,##);};
        


//=======================================================
// For

for_st : FOR_RW   for_header LEFT_BRACE! body RIGHT_BRACE!
        {## = #( #[FOR_ST, "FOR_ST"] ,##);};

for_header : LEFT_PAREN! for_init for_cond SEMICOLON! INT_LITERAL RIGHT_PAREN!
             {## = #( #[FOR_HEADER, "FOR_HEADER"] ,##);};

for_init : assign;

for_cond : expr;


//=======================================================
// If

if_st : IF_RW  LEFT_PAREN! if_cond RIGHT_PAREN! LEFT_BRACE! body RIGHT_BRACE!  (else_st)?
        {## = #( #[IF_ST, "IF_ST"] ,##);};
        
if_cond : expr;
 

else_st: ELSE_RW  LEFT_BRACE! body RIGHT_BRACE!
        {## = #( #[ELSE_ST, "ELSE_ST"] ,##);};
 
//=======================================================
//Expressions 


expr          :  expAND  (OR_RW^ expAND)*;

expAND        :  expCons (AND_RW^ expCons)*;

expCons       :  expSum  ((EQUAL^| 
                           LE_EQUAL^|
                           GR_EQUAL^|
                           NOT_EQUAL^|
                           GR^|
                           LE^) expSum)* ; 

expSum        :  expProduct  ((PLUS^| 
                               SUB^) expProduct)* ; 

expProduct    :  expMinus ((STAR^ | 
                            SLASH^) expMinus)* ;  

expMinus      : (SUB! expNot)
                { ##=#(#[UN_MINUS, "UN_MINUS"], ##) ;} 
              | ((PLUS!)? expNot)
              ;

expNot: (NOT_RW^)? (access|lit|(LEFT_PAREN! expr RIGHT_PAREN!));


lit: REAL_LITERAL|INT_LITERAL;


access : IDENT (DOT IDENT | access_param)*  
         {## = #( #[ACCESS, "ACCESS"] ,##);};

access_param: LEFT_PAREN! (IDENT (COMMA IDENT)*)? RIGHT_PAREN!
              {## = #( #[ACCESS_PARAM, "ACCESS_PARAM"] ,##);};

//=======================================================
// in-out

read : READ_RW LEFT_PAREN! IDENT RIGHT_PAREN! SEMICOLON! ;

print : PRINT_RW LEFT_PAREN! STRING_LITERAL COMMA! IDENT RIGHT_PAREN! SEMICOLON! ;  





